import { EditorState, getDefaultKeyBinding, Editor, Modifier } from 'draft-js';
import React, { Component, useEffect } from 'react';
import { instanceOf, func, array, bool, shape, number } from 'prop-types';
import { createPortal } from 'react-dom';

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var KeyCodes = {
  ARROW_UP: 38,
  ARROW_DOWN: 40
};
var Commands = {
  ARROW_UP: 'omni-editor-arrow-up',
  ARROW_DOWN: 'omni-editor-arrow-down',
  SPLIT_BLOCK: 'split-block'
};
var OmniEditorKeyCommandMap = {};
OmniEditorKeyCommandMap[KeyCodes.ARROW_UP] = Commands.ARROW_UP;
OmniEditorKeyCommandMap[KeyCodes.ARROW_DOWN] = Commands.ARROW_DOWN;

var OmniEditor =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(OmniEditor, _Component);

  function OmniEditor(props) {
    var _this;

    _this = _Component.call(this, props) || this;

    _this.componentDidMount = function () {
      if (_this.props.shouldFocus) {
        _this.editor.focus();
      }
    };

    _this.onChange = function (editorState) {
      _this.props.onChange(editorState); // Wait for the DOM to be re-rendered from the editor state change before updating the plugins.


      setTimeout(function () {
        _this.setState({
          plugins: _this.state.plugins.map(function (plugin) {
            return plugin.onChange(editorState);
          })
        });
      }, 0);
    };

    _this.myKeyBindingFn = function (event) {
      var shouldTriggerCustomKeyCommand = false;

      _this.state.plugins.forEach(function (plugin) {
        var shouldTrigger = plugin.shouldTriggerCustomKeyCommand();

        if (shouldTrigger) {
          shouldTriggerCustomKeyCommand = true;
        }
      });

      if (!shouldTriggerCustomKeyCommand) {
        return getDefaultKeyBinding(event);
      }

      if (OmniEditorKeyCommandMap[event.keyCode]) {
        return OmniEditorKeyCommandMap[event.keyCode];
      }

      return getDefaultKeyBinding(event);
    };

    _this.handleKeyCommand = function (command, editorState) {
      var isHandled = false;

      var updatedPlugins = _this.state.plugins.map(function (plugin) {
        var _plugin$handleKeyComm = plugin.handleKeyCommand(command, editorState),
            didPluginHandleCommand = _plugin$handleKeyComm.didPluginHandleCommand,
            updatedPlugin = _plugin$handleKeyComm.updatedPlugin,
            updatedEditorState = _plugin$handleKeyComm.updatedEditorState;

        if (didPluginHandleCommand) {
          isHandled = true;
        }

        if (updatedEditorState) {
          _this.props.onChange(updatedEditorState);
        }

        return updatedPlugin;
      });

      if (isHandled) {
        _this.setState({
          plugins: updatedPlugins
        });
      }

      return isHandled ? 'handled' : 'not-handled';
    };

    _this.render = function () {
      return React.createElement("div", {
        className: "OmniEditor"
      }, _this.state.plugins.map(function (plugin) {
        return plugin.getPortalElement(_this.props.editorState, _this.props.onChange);
      }), React.createElement(Editor, {
        ref: function ref(node) {
          return _this.editor = node;
        },
        editorState: _this.props.editorState,
        onChange: _this.onChange,
        keyBindingFn: _this.myKeyBindingFn,
        handleKeyCommand: _this.handleKeyCommand
      }));
    };

    _this.state = {
      plugins: props.plugins
    };
    return _this;
  }

  return OmniEditor;
}(Component);

OmniEditor.propTypes = {
  editorState:
  /*#__PURE__*/
  instanceOf(EditorState).isRequired,
  onChange: func.isRequired,
  plugins: array,
  shouldFocus: bool
};

var MentionSuggestion = function MentionSuggestion(props) {
  var suggestionClassName = [];

  if (props.suggestion.active) {
    suggestionClassName.push('active');
  }

  var onMouseDown = function onMouseDown(event) {
    event.preventDefault();
  };

  var onMouseUp = function onMouseUp() {
    props.onSuggestion(props.suggestion);
  };

  return React.createElement("li", {
    className: suggestionClassName.join(' '),
    onMouseDown: onMouseDown,
    onMouseUp: onMouseUp
  }, props.suggestionView(props.suggestion));
};

var MentionSuggestionsDropdown = function MentionSuggestionsDropdown(props) {
  var element = React.useRef(document.createElement('div'));
  useEffect(function () {
    var currentElement = element.current;
    document.body.appendChild(currentElement);
    return function removeElement() {
      currentElement.remove();
    };
  }, [element]);
  var dropdownPosition = {
    top: props.position.top,
    left: props.position.left
  };
  var mentionSuggestionsDropdown = React.createElement("ul", {
    className: "MentionSuggestionsDropdown",
    style: dropdownPosition
  }, props.suggestions.map(function (suggestion, index) {
    return React.createElement(MentionSuggestion, {
      key: index,
      suggestion: suggestion,
      suggestionView: props.suggestionView,
      onSuggestion: props.onSuggestion
    });
  }));
  return createPortal(mentionSuggestionsDropdown, element.current);
};

MentionSuggestionsDropdown.propTypes = {
  suggestions: array.isRequired,
  suggestionView: func.isRequired,
  onSuggestion: func.isRequired,
  position:
  /*#__PURE__*/
  shape({
    left: number.isRequired,
    top: number.isRequired
  }).isRequired
};

var MentionPlugin = function MentionPlugin(trigger, suggestions, suggestionView, suggestionFilter) {
  var _this = this;

  this.hasEntityAtSelection = function (editorState) {
    var selection = editorState.getSelection();

    if (!selection.getHasFocus()) {
      return false;
    }

    var contentState = editorState.getCurrentContent();
    var block = contentState.getBlockForKey(selection.getAnchorKey());
    return !!block.getEntityAt(selection.getAnchorOffset() - 1);
  };

  this.getTriggerRange = function (editorState) {
    var selection = window.getSelection();

    if (!selection) {
      return null;
    }

    if (selection.rangeCount === 0) {
      return null;
    }

    if (_this.hasEntityAtSelection(editorState)) {
      return null;
    }

    var range = selection.getRangeAt(0);
    var text = range.startContainer.textContent;

    if (!text) {
      return null;
    } // Remove text that appears after the cursor..


    text = text.substring(0, range.startOffset);
    var index = text.lastIndexOf(_this.trigger);

    if (index === -1) {
      return null;
    }

    text = text.substring(index).substring(1, text.length).toLowerCase();
    return {
      selection: selection,
      text: text,
      start: index,
      end: range.startOffset
    };
  };

  this.getSuggestionState = function (editorState) {
    var range = _this.getTriggerRange(editorState);

    if (!range) {
      return null;
    }

    var tempRange = range.selection.getRangeAt(0).cloneRange();
    tempRange.setStart(tempRange.startContainer, range.start);
    var rangeRect = tempRange.getBoundingClientRect();
    var _ref = [rangeRect.left, rangeRect.bottom],
        left = _ref[0],
        top = _ref[1];
    return {
      text: range.text,
      position: {
        left: left,
        top: top
      },
      indexOfSelectedSuggestion: 0
    };
  };

  this.onChange = function (editorState) {
    _this.state = _this.getSuggestionState(editorState);
    return _this;
  };

  this.shouldTriggerCustomKeyCommand = function () {
    if (_this.state === null) {
      return false;
    }

    return true;
  };

  this.handleKeyCommand = function (command, editorState) {
    if (_this.state === null) {
      return {
        didPluginHandleCommand: false,
        updatedPlugin: _this
      };
    }

    var filteredSuggestions = _this.suggestions.filter(function (suggestion) {
      return _this.suggestionFilter(suggestion, _this.state);
    });

    if (command === Commands.ARROW_DOWN) {
      if (_this.state.indexOfSelectedSuggestion === filteredSuggestions.length - 1) {
        _this.state.indexOfSelectedSuggestion = 0;
      } else {
        _this.state.indexOfSelectedSuggestion++;
      }
    } else if (command === Commands.ARROW_UP) {
      if (_this.state.indexOfSelectedSuggestion === 0) {
        _this.state.indexOfSelectedSuggestion = filteredSuggestions.length - 1;
      } else {
        _this.state.indexOfSelectedSuggestion--;
      }
    } else if (command === Commands.SPLIT_BLOCK) {
      var updatedEditorState = _this.addMentionToEditorState(editorState, filteredSuggestions[_this.state.indexOfSelectedSuggestion]);

      _this.state = null;
      return {
        didPluginHandleCommand: true,
        updatedPlugin: _this,
        updatedEditorState: updatedEditorState
      };
    } else {
      return {
        didPluginHandleCommand: false,
        updatedPlugin: _this
      };
    }

    return {
      didPluginHandleCommand: true,
      updatedPlugin: _this
    };
  };

  this.addMentionToEditorState = function (editorState, suggestedMention) {
    console.log(suggestedMention);

    if (!suggestedMention) {
      return editorState;
    } // Get current selection from the editor state.


    var selection = editorState.getSelection(); // Set all variables required to calculate the start and end position for our mention.

    var anchorKey = selection.getAnchorKey();
    var anchorOffset = selection.getAnchorOffset();
    var currentContent = editorState.getCurrentContent().createEntity('MENTION', 'IMMUTABLE', suggestedMention);
    var currentBlock = currentContent.getBlockForKey(anchorKey);
    var blockText = currentBlock.getText(); // Based on the trigger position calculate start and end index of our mention.

    var selectionText = blockText.substring(0, anchorOffset);
    var anchorOffsetOfMention = _this.trigger.length === 0 ? 0 : selectionText.lastIndexOf(_this.trigger);
    var focusOffsetOfMention = selectionText.length; // Create our entity key and modify the selection to contain the mention text.

    var entityKey = currentContent.getLastCreatedEntityKey();
    var mentionTextSelection = selection.merge({
      anchorOffset: anchorOffsetOfMention,
      focusOffset: focusOffsetOfMention
    }); // Replace the current text inside the selection state with the mentioned persons text.

    var contentStateWithMentionEntity = Modifier.replaceText(currentContent, mentionTextSelection, "@" + suggestedMention.name, undefined, entityKey); // Add a space after the mention entity if it's the last item in the block.

    var blockKey = mentionTextSelection.getAnchorKey();
    var blockSize = currentContent.getBlockForKey(blockKey).getLength();

    if (blockSize === focusOffsetOfMention) {
      contentStateWithMentionEntity = Modifier.insertText(contentStateWithMentionEntity, contentStateWithMentionEntity.getSelectionAfter(), ' ');
    } // Return the updated editor state containing the new mention entity.


    return EditorState.forceSelection(EditorState.push(editorState, contentStateWithMentionEntity, 'insert-mention'), contentStateWithMentionEntity.getSelectionAfter());
  };

  this.getPortalElement = function (editorState, setEditorState) {
    if (_this.state === null) {
      return null;
    }

    var filteredSuggestions = _this.suggestions.filter(function (suggestion) {
      return _this.suggestionFilter(suggestion, _this.state);
    }).map(function (suggestion, index) {
      if (_this.state === null) {
        return suggestion;
      }

      suggestion.active = _this.state.indexOfSelectedSuggestion === index;
      return suggestion;
    });

    var onSuggestion = function onSuggestion(suggestion) {
      setEditorState(_this.addMentionToEditorState(editorState, suggestion));
      _this.state = null;
    };

    return React.createElement(MentionSuggestionsDropdown, {
      key: "mention-suggestions",
      suggestionView: _this.suggestionView,
      suggestions: filteredSuggestions,
      position: _this.state.position,
      onSuggestion: onSuggestion
    });
  };

  this.trigger = trigger;
  this.suggestions = suggestions;
  this.suggestionView = suggestionView;
  this.suggestionFilter = suggestionFilter;
  this.state = null;
};

export { MentionPlugin, OmniEditor };
//# sourceMappingURL=draft-js-omni-editor.esm.js.map
